/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bcee.jobs;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;

import com.gargoylesoftware.htmlunit.WebClient;

import de.frankmuenster.mahoe.banks.bcee.Bcee;
import de.frankmuenster.mahoe.banks.bcee.web.BceeBankMessages;
import de.frankmuenster.mahoe.banks.bcee.web.BceeWebAccountInformation;
import de.frankmuenster.mahoe.banks.bcee.web.BceeWebSite;
import de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.PasswdCache;
import de.frankmuenster.mahoe.utils.TransactionSaver;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Der Job der die eigentliche Arbeit macht.
 *
 * @author Frank Mnster
 */
public class BceeSynchronizeJobKontoauszug
        extends AbstractMahoeSynchronizeJobKontoauszug<WebClient, BceeWebAccountInformation, BceeBankMessages> {

    /**
     * Standartkonstruktor
     */
    public BceeSynchronizeJobKontoauszug() {

        super();
    }

    /**
     * @see AbstractMahoeSynchronizeJobKontoauszug#getWebSite()
     */
    @Override
    protected void getWebSite() throws WebSiteException, ApplicationException {

        this.webSite = BceeWebSite.getInstance(this.konto);
    }

    /**
     * @see AbstractMahoeSynchronizeJobKontoauszug#handleTransactions()
     */
    @Override
    protected void handleTransactions() throws WebSiteException, ApplicationException {

        // Vorgemerkte Umstze
        this.monitor.setStatusText(I18N.tr("BceeSynchronizeJobKontoauszug.getPreBooked", //$NON-NLS-1$
                this.accountInfo.getLastTransactionDateAsString()));
        final List<Umsatz> preBooked = this.webSite.getPreBookedTransactions();

        // Debug-Ausgabe
        this.debugLogTransactions(preBooked);

        // Speicher vorgemerkte Umstze
        this.monitor.setStatusText(I18N.tr("BceeSynchronizeJobKontoauszug.savingPreBooked")); //$NON-NLS-1$
        final int preBookedCount = TransactionSaver.savePreBooked(this.konto, preBooked, false);
        this.monitor.setStatusText(
                I18N.tr("BceeSynchronizeJobKontoauszug.savedPreBooked", Integer.toString(preBookedCount))); //$NON-NLS-1$
        this.monitor.setPercentComplete(40);
        Logger.info(I18N.tr("BceeSynchronizeJobKontoauszug.savedPreBooked", Integer.toString(preBookedCount))); //$NON-NLS-1$

        // Gebuchte Umstze
        this.monitor.setStatusText(
                I18N.tr("SynchronizeJobKontoauszug.getBooked", this.accountInfo.getLastTransactionDateAsString())); //$NON-NLS-1$
        final List<Umsatz> booked = this.webSite.getBookedTransactions();
        // Debug-Ausgabe
        this.debugLogTransactions(booked);

        // Speicher gebuchte Umstze
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savingBooked")); //$NON-NLS-1$
        final int bookedCount = TransactionSaver.saveBooked(this.konto, booked);
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$
        this.monitor.setPercentComplete(50);
        Logger.info(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$

        // Saldo/verfgbarer Betrag setzten
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.updateSaldo")); //$NON-NLS-1$
        TransactionSaver.saveSaldo(this.konto, this.accountInfo.getSaldo());
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.newSaldoIs", //$NON-NLS-1$
                this.accountInfo.getSaldo()));
        if (this.accountInfo.getCreditLimit() == null) {
            TransactionSaver.saveSaldoAvailable(this.konto, BigDecimal.ZERO);
            this.monitor.setStatusText(I18N.tr("BceeSynchronizeJobKontoauszug.creditLimitMissing")); //$NON-NLS-1$
        } else {

            final BigDecimal available = BigDecimal.ZERO
                    .max(this.accountInfo.getSaldo().add(BigDecimal.valueOf(this.accountInfo.getCreditLimit())));
            TransactionSaver.saveSaldoAvailable(this.konto, available);
            this.monitor.setStatusText(I18N.tr("BceeSynchronizeJobKontoauszug.newAvailable", //$NON-NLS-1$
                    available));
            this.monitor.setPercentComplete(55);
        }
    }

    /**
     * @see AbstractMahoeSynchronizeJobKontoauszug#login()
     */
    @Override
    protected void login() throws WebSiteException, ApplicationException {

        final char[] password = PasswdCache.getPassword(Bcee.NAME, this.kundennummer);
        try {

            final String otp = BceeSynchronizeJobKontoauszugUtils.getOtp();
            this.webSite.login(password, otp);
        } catch (final WebSiteException e) {
            if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)) {
                // Passwort zurcksetzten
                PasswdCache.revoke(Bcee.NAME, this.kundennummer);
            }
            throw e;
        }
        Arrays.fill(password, ' ');

    }

}
