/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bcee.web;

import java.math.BigDecimal;

import de.frankmuenster.mahoe.web.AbstractWebAccountInformation;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * BCEE spezifisch Kontoinformationen
 *
 * @author Frank Mnster
 */
public class BceeWebAccountInformation extends AbstractWebAccountInformation {

    /**
     * Der Betrag der Kreditlinie
     */
    private Double creditLimit;

    /**
     * Currency
     */
    private String currency;

    /**
     * Sollzinssatz der Kreditlinie
     */
    private Double interestRate;

    /**
     * Standardkonstruktor
     */
    public BceeWebAccountInformation() {

        super();
    }

    /**
     * @return the creditLimit
     */
    public Double getCreditLimit() {

        return this.creditLimit;
    }

    /**
     * @return the currency
     */
    public String getCurrency() {

        return this.currency;
    }

    /**
     * @return the interestRate
     */
    public Double getInterestRate() {

        return this.interestRate;
    }

    /**
     * @param creditLimit
     *            the creditLimit to set
     */
    public void setCreditLimit(final Double creditLimit) {

        this.creditLimit = creditLimit;
        // Verfgbarer Betrag aktualisieren
        if (this.getSaldo() != null) {
            final BigDecimal available = BigDecimal.ZERO.max(this.getSaldo().add(BigDecimal.valueOf(creditLimit))); // NOPMD
            this.setSaldoAvailable(available);
        }

    }

    /**
     * @param currency
     *            the currency to set
     */
    public void setCurrency(final String currency) {

        this.currency = currency;
    }

    /**
     * @param interestRate
     *            the interestRate to set
     */
    public void setInterestRate(final double interestRate) {

        this.interestRate = interestRate;
    }

    /**
     * @see AbstractWebAccountInformation#saveInformationInAccount(ProgressMonitor)
     */
    @Override
    protected void saveBankSpecificInfo(final ProgressMonitor monitor) throws ApplicationException {

        // Fr BCEE ist hier nix zutun
    }

}
