/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bos.web; // NOPMD

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.openqa.selenium.By;
import org.openqa.selenium.NoSuchSessionException;
import org.openqa.selenium.SessionNotCreatedException;
import org.openqa.selenium.TimeoutException;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;

import de.frankmuenster.mahoe.banks.bos.BankOfScotland;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.AbstractSeleniumWebSite;
import de.frankmuenster.mahoe.web.MahoeKontoauszugPDF;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * @author Frank Mnster
 */
public final class BosWebSite extends AbstractSeleniumWebSite<BosWebAccountInformation, BosBankMessages> {

    /**
     * I18N constant BosWebSite.accountNumberNotSet
     */
    private static final String I18N_BOS_WEB_SITE_ACCOUNT_NUMBER_NOT_SET = "BosWebSite.accountNumberNotSet"; // NOPMD //$NON-NLS-1$

    /**
     * Our instance
     */
    private static BosWebSite webSite;

    /**
     * The account name
     */
    private final String userName;

    /**
     * Returns the instance of the BosWebsite. Creates a new one, if none exists
     * or the users nam eis different.
     *
     * @param konto
     *            the account to create the instance for
     * @throws WebSiteException
     * @throws ApplicationException
     * @return the instance of the BosWebsite for this account/user name
     */
    public static BosWebSite getInstance(final Konto konto) throws WebSiteException, ApplicationException {

        String userName = null;
        try {
            userName = konto.getKundennummer();
        } catch (final RemoteException e) {
            final String msg = I18N.tr("AbstractWebSite.loginMissing"); // NOPMD //$NON-NLS-1$
            Logger.error(msg, e);
            throw new ApplicationException(msg, e);
        }
        if (BosWebSite.webSite == null) {
            BosWebSite.webSite = new BosWebSite(konto, userName);
        } else {
            // Ist der Benutzername der selbe
            if (BosWebSite.webSite.userName.equals(userName)) {
                // Aktualisiere Startdatum der Umsatzabfrage und lsche
                // Umsatzliste
                final Date lastTransacDate = Utils.getLastUmsatzDate(konto, BosWebSite.webSite.getStartdate());
                BosWebSite.webSite.accountInfo.setLastTransactionDate(lastTransacDate);
            } else {
                BosWebSite.webSite = new BosWebSite(konto, userName);
            }
        }
        return BosWebSite.webSite;

    }

    /**
     * Creates a new WebSite instance to communicate with the bank
     *
     * @param konto
     * @param accountName
     * @throws WebSiteException
     */
    private BosWebSite(final Konto konto, final String accountName) throws WebSiteException {

        super(konto);
        this.userName = accountName;

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#createSepaTransfer(de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung)
     */
    @Override
    public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung)
            throws WebSiteException, ApplicationException {

        // Will not be supported
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getAccountInformation()
     */
    @Override
    public BosWebAccountInformation getAccountInformation() throws WebSiteException, ApplicationException {

        // first check if driver available
        this.checkSeleniumDriverSet(this.webClient);

        try {
            final String accountNumber = this.accountInfo.getKonto().getKontonummer();
            return BosWebSiteHelper.getAccountInfo(this.webClient, this.accountInfo, accountNumber);
        } catch (final RemoteException e) {
            Logger.error(I18N.tr(BosWebSite.I18N_BOS_WEB_SITE_ACCOUNT_NUMBER_NOT_SET));
            throw new ApplicationException(I18N.tr(BosWebSite.I18N_BOS_WEB_SITE_ACCOUNT_NUMBER_NOT_SET), e);
        }

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getAccountSheets(de.frankmuenster.mahoe.web.WebAccountInformation)
     */
    @Override
    public List<MahoeKontoauszugPDF> getAccountSheets(final BosWebAccountInformation accountInfo)
            throws WebSiteException {

        this.checkSeleniumDriverSet(this.webClient);

        final BosAccountStatementFinder accountSheets = new BosAccountStatementFinder(accountInfo, this.webClient);
        accountSheets.findStatements();

        return accountSheets.getSheets();
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getBookedTransactions()
     */
    @Override
    public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {

        // first check if driver available
        this.checkSeleniumDriverSet(this.webClient);

        try {
            final String accountNumber = this.accountInfo.getKonto().getKontonummer();
            return BosWebSiteHelper.getBookedTransactions(this.webClient, this.accountInfo, accountNumber);
        } catch (final RemoteException e) {
            Logger.error(I18N.tr(BosWebSite.I18N_BOS_WEB_SITE_ACCOUNT_NUMBER_NOT_SET));
            throw new ApplicationException(I18N.tr(BosWebSite.I18N_BOS_WEB_SITE_ACCOUNT_NUMBER_NOT_SET), e);
        }
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getPreBookedTransactions()
     */
    @Override
    public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {

        // Doesn't exists by BOS
        return new ArrayList<>();
    }

    /**
     * @see de.frankmuenster.mahoe.web.AbstractWebSite#getStartdate()
     */
    @Override
    protected Calendar getStartdate() {

        // Setzte das startDatum auf Januar 2016
        final Calendar startCalendar = Calendar.getInstance();
        startCalendar.set(2016, 0, 1);
        return startCalendar;
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#isStillLoggedIn()
     */
    @Override
    public boolean isStillLoggedIn() throws WebSiteException {

        // first check if driver available
        this.checkSeleniumDriverSet(this.webClient);

        String check = StringUtils.EMPTY;
        final WebDriverWait wait = new WebDriverWait(this.webClient, 1);
        try {
            this.webClient.switchTo().defaultContent();
            wait.until(ExpectedConditions.frameToBeAvailableAndSwitchToIt(By.name("frame_top"))); //$NON-NLS-1$
            wait.until(ExpectedConditions.elementToBeClickable(By.name("btnContinue"))); //$NON-NLS-1$
            check = this.webClient.getPageSource();
        } catch (TimeoutException | NoSuchSessionException | SessionNotCreatedException e) {
            Logger.debug("Timeout aufgetretten --> also nicht angemeldet."); //$NON-NLS-1$
        }
        // Check if we are logged in
        return check.contains("Abmelden"); //$NON-NLS-1$

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#login(char[], java.lang.String[])
     */
    @Override
    public void login(final char[] password, final String... loginParams) throws WebSiteException {

        // first check if driver available
        this.checkSeleniumDriverSet(this.webClient);

        Logger.debug("Starte mit Login fr Benutzername {0}" + this.userName); //$NON-NLS-1$

        if (loginParams.length != 0) {
            throw new WebSiteException(I18N.tr("AbstractBankWebSite.loginParameterNumberWrong"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_ERROR);
        }
        // check if driver is still active

        try {
            this.webClient.getTitle();
        } catch (final NoSuchSessionException | SessionNotCreatedException e) {
            this.getWebClient();
        }
        // Perform login
        BosWebSiteHelper.login(this.webClient, this.userName, java.nio.CharBuffer.wrap(password));

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#logoff()
     */
    @Override
    public void logoff() throws WebSiteException {

        // first check if driver available
        this.checkSeleniumDriverSet(this.webClient);

        // perform logout
        BosWebSiteHelper.logoff(this.webClient);

        // finally save the cookies
        saveCookies(BankOfScotland.COOKIE_FILE);

    }

}
