/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bsh.assistant;

import de.frankmuenster.mahoe.assistant.CreateInfo;
import de.frankmuenster.mahoe.banks.bsh.SchwaebischHall;
import de.frankmuenster.mahoe.banks.bsh.json.Account;

/**
 * Bentigte Informationen zum Erstellen eines Bsh Kontos
 *
 * @author Frank Mnster
 */
public class BshCreateInfo extends CreateInfo {

    /**
     * Account JSON
     */
    private Account account;

    /**
     * Ask always Passwort
     */
    private boolean alwaysAskForPW;

    /**
     * Show password for debug purpose
     */
    private boolean debugShowPW;

    /**
     * Kundennummer
     */
    private String kdNummer;

    /**
     * Standardkonstruktor
     */
    public BshCreateInfo() {
        super();
    }

    /**
     * @return das Account-Object
     */
    public Account getAccount() {

        return this.account;
    }

    /**
     * @return the blz
     */
    public String getBlzDz() {

        return SchwaebischHall.BLZ_DZ;
    }

    /**
     * @return the bsn
     */
    public String getBsn() {

        return this.account.getBsn();
    }

    /**
     * @return die Hauptnummer des Vertrages
     */
    public String getCoreId() {

        return this.account.getCoreId();
    }

    /**
     * @return the kdNummer
     */
    public String getKdNummer() {

        return this.kdNummer;
    }

    /**
     * @return coreId+Buchstabe+Nummer
     */
    public String getLetterNumber() {

        return this.account.getContractLetter() + this.account.getContractNumber();
    }

    /**
     * @return the vtn
     */
    public String getVtn() {

        return this.account.getVtn();
    }

    /**
     * @return the alwaysAskForPW
     */
    public boolean isAlwaysAskForPW() {

        return this.alwaysAskForPW;
    }

    /**
     *
     * @return the debugShowPW
     */
    public boolean isDebugShowPW() {
        return this.debugShowPW;
    }

    /**
     * @param account
     *            the account to set
     */
    public void setAccount(final Account account) {

        this.account = account;
    }

    /**
     * @param alwaysAskForPW
     *            the alwaysAskForPW to set
     */
    public void setAlwaysAskForPW(final boolean alwaysAskForPW) {

        this.alwaysAskForPW = alwaysAskForPW;
    }

    /**
     *
     * @param debugShowPW
     *            the debugShowPW to set
     */
    public void setDebugShowPW(final boolean debugShowPW) {
        this.debugShowPW = debugShowPW;
    }

    /**
     * @param kdNummer
     *            the kdNummer to set
     */
    public void setKdNummer(final String kdNummer) {

        this.kdNummer = kdNummer;
    }

}
