
package de.frankmuenster.mahoe.banks.bsh.json.deserializer;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonNode;

import de.frankmuenster.mahoe.utils.I18N;

/**
 * Deserializer fr das Datum der BSH
 *
 * @author Frank Mnster/Quirin Huber
 */
public class BshDateDeserializer extends JsonDeserializer<Date> {

    /**
     * Datumsformat der BSH
     */
    private static final String PATTERN = "dd.MM.yyyy"; //$NON-NLS-1$

    /**
     * Standardkonstruktor
     */
    public BshDateDeserializer() {
        super();
    }

    /**
     * @see JsonDeserializer#deserialize(JsonParser, DeserializationContext)
     */
    @Override
    public Date deserialize(final JsonParser jsonParser, final DeserializationContext context) throws IOException {

        final SimpleDateFormat sdf = new SimpleDateFormat(BshDateDeserializer.PATTERN, Locale.GERMAN);

        final ObjectCodec objectCodec = jsonParser.getCodec();
        final JsonNode node = objectCodec.readTree(jsonParser);
        final String textValue = node.asText();
        Date parse;
        try {
            parse = sdf.parse(textValue);
        } catch (final ParseException e) {
            throw new JsonParseException(jsonParser, I18N.tr("BshDateDeserializer.parseDateFailed"), e); //$NON-NLS-1$
        }
        return parse;

    }
}
