/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bsh.web;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.web.AbstractWebAccountInformation;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Bean mit Kontoinformationen von der Webseite
 *
 * @author Frank Mnster
 * @version 1.0
 */
public class BshWebAccountInformation extends AbstractWebAccountInformation {

    /**
     * Die BSN Nummer
     */
    private String bsn;

    /**
     * Die Vertragsbuchstabe
     */
    private String contractLetter;

    /**
     * Die Vertragsnummer
     */
    private String contractNumber;

    /**
     * Debug show password
     */
    private boolean showPassword;

    /**
     * Der Vertragstyp
     */
    private String type;

    /**
     * Die VTN Nummer
     */
    private String vtn;

    /**
     * Standardkonstruktor
     */
    public BshWebAccountInformation() {

        super();
    }

    /**
     * @return the bsn
     */
    public String getBsn() {

        return this.bsn;
    }

    /**
     * @return the contractLetter
     */
    public String getContractLetter() {

        return this.contractLetter;
    }

    /**
     * @return the contractNumber
     */
    public String getContractNumber() {

        return this.contractNumber;
    }

    /**
     * @return the type
     */
    public String getType() {

        return this.type;
    }

    /**
     * @return the vtn
     */
    public String getVtn() {

        return this.vtn;
    }

    /**
     * @return the showPassword
     */
    public boolean isShowPassword() {

        return this.showPassword;
    }

    /**
     * @param bsn
     *            the bsn to set
     */
    public void setBsn(final String bsn) {

        this.bsn = bsn;
    }

    /**
     * @param contractLetter
     *            the contractLetter to set
     */
    public void setContractLetter(final String contractLetter) {

        this.contractLetter = contractLetter;
    }

    /**
     * @param contractNumber
     *            the contractNumber to set
     */
    public void setContractNumber(final String contractNumber) {

        this.contractNumber = contractNumber;
    }

    /**
     * @param showPassword
     *            the showPassword to set
     */
    public void setShowPassword(final boolean showPassword) {

        this.showPassword = showPassword;
    }

    /**
     * @param type
     *            the type to set
     */
    public void setType(final String type) {

        this.type = type;
    }

    /**
     * @param vtn
     *            the vtn to set
     */
    public void setVtn(final String vtn) {

        this.vtn = vtn;
    }

    /**
     * @see AbstractWebAccountInformation#saveBankSpecificInfo(de.willuhn.util.ProgressMonitor)
     */
    @Override
    protected void saveBankSpecificInfo(final ProgressMonitor monitor) throws ApplicationException {

        // vtn-Nummer
        if (this.vtn != null) {
            monitor.setStatusText(I18N.tr("BshWebAccountInformation.saveVtn")); //$NON-NLS-1$
            try {
                final String ktoVTN = this.konto.getMeta(MahoeSynchronizeBackend.BSH_VTN, null);
                if (StringUtils.isEmpty(ktoVTN)) {
                    this.konto.setMeta(MahoeSynchronizeBackend.BSH_VTN, this.vtn);
                }
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("BshWebAccountInformation.saveVtnFailed"), e); //$NON-NLS-1$
                throw new ApplicationException(I18N.tr("BshWebAccountInformation.saveVtnFailed"), e); //$NON-NLS-1$
            }
        }

        // Unterkonto
        final StringBuilder unterKto = new StringBuilder();
        unterKto.append(this.getContractLetter()).append(this.getContractNumber()).append('%').append(this.getType());
        try {
            if (!unterKto.toString().equals(this.konto.getUnterkonto())) {
                unterKto.setLength(30);
                this.konto.setUnterkonto(unterKto.toString());
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("BshWebAccountInformation.saveSubAccountFailed"), e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("BshWebAccountInformation.saveSubAccountFailed"), e); //$NON-NLS-1$
        }
    }
}
