/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.santander.assistant;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.assistant.AbstractAccountMigrator;
import de.frankmuenster.mahoe.assistant.AccountMigrator;
import de.frankmuenster.mahoe.banks.santander.SantanderBank;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.plugin.Plugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Der Migrator fr existierende Santander Konten
 *
 * @author Frank Mnster
 */
public class SantanderAccountMigrator extends AbstractAccountMigrator implements AccountMigrator {

    /**
     * Der alte Plugin Name
     */
    private static final String OLD_PLUGIN_NAME = "hibiscus.script.santander_consumer"; //$NON-NLS-1$

    /**
     * Standardkonstruktor
     */
    public SantanderAccountMigrator() {
        super();
    }

    /**
     * @see de.frankmuenster.mahoe.assistant.AbstractAccountMigrator#migrate
     *      (de.willuhn.jameica.plugin.Plugin)
     */
    @Override
    public void migrate(final Plugin plugin) throws ApplicationException {

        int countMigrated = 0;
        // Da wir kein altes Plugin haben, mach wir es etwas anders
        if (SantanderAccountMigrator.OLD_PLUGIN_NAME.equals(plugin.getManifest().getName())) {
            Logger.info("Santander-Migrator gestartet."); //$NON-NLS-1$
            try {
                final HBCIDBService hcbiService = de.willuhn.jameica.hbci.Settings.getDBService();
                final DBIterator<Konto> accounts = hcbiService.createList(Konto.class);
                while (accounts.hasNext()) {
                    final Konto account = accounts.next();
                    if (SantanderBank.BLZ.equals(account.getBLZ())
                            && !MahoeSynchronizeBackend.BACKEND_NAME.equals(account.getBackendClass())) {
                        this.migrateAccount(account);
                        countMigrated++;
                    }
                }
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("SantanderAccountMigrator.readAccountsFailed", e.getMessage()), e); //$NON-NLS-1$
            }
            if (countMigrated > 0) {
                final StatusBarMessage statusBarMsg = new StatusBarMessage(
                        I18N.tr("SantanderAccountMigrator.accountMigratedSuccessfully"), StatusBarMessage.TYPE_SUCCESS); //$NON-NLS-1$
                Application.getMessagingFactory().sendMessage(statusBarMsg);
            }
            // Plugin entfernen
            this.uninstallPlugin(plugin);
        }
    }

    /**
     * @see de.frankmuenster.mahoe.assistant.AbstractAccountMigrator#update(de.willuhn.jameica.hbci.rmi.Konto)
     */
    @Override
    public void update(final Konto account) throws ApplicationException {

        try {
            if (SantanderBank.BLZ.equals(account.getBLZ())) {
                // Kredit oder GIRO?
                final String oldVersion = account.getMeta(MahoeSynchronizeBackend.KONTO_VERSION, "0"); //$NON-NLS-1$
                if (KontoType.GIRO.equals(KontoType.find(account.getAccountType()))) {
                    // GIRO/Tagesgeld
                    switch (oldVersion) { // NOPMD
                    case "0": //$NON-NLS-1$
                        // Update to 1
                        Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "1")); //$NON-NLS-1$
                        account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "1"); //$NON-NLS-1$
                    case "1": //$NON-NLS-1$
                        Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "2")); //$NON-NLS-1$
                        Utils.removePdfMeta(account);
                        account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "2"); //$NON-NLS-1$
                    case "2": //$NON-NLS-1$
                        Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "3")); //$NON-NLS-1$
                        account.setUnterkonto(StringUtils.EMPTY);
                        account.setMeta(MahoeSynchronizeBackend.BANK_NAME, SantanderBank.NAME);
                        account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "3"); //$NON-NLS-1$
                    default:
                        break;
                    }
                } else {
                    // Kreditkarten
                    switch (oldVersion) { // NOPMD
                    case "0": //$NON-NLS-1$
                        // Update to 1
                        Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "1")); //$NON-NLS-1$
                        account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "1"); //$NON-NLS-1$
                    case "1": //$NON-NLS-1$
                        Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "2")); //$NON-NLS-1$
                        account.setMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT, StringUtils.EMPTY);
                        account.setMeta(MahoeSynchronizeBackend.CREATE_CLEARING, StringUtils.EMPTY);
                        account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "2"); //$NON-NLS-1$
                    case "2": //$NON-NLS-1$
                        Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "3")); //$NON-NLS-1$
                        account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "3"); //$NON-NLS-1$
                    case "3": //$NON-NLS-1$
                        Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "3")); //$NON-NLS-1$
                        Utils.removePdfMeta(account);
                        account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "4"); //$NON-NLS-1$
                    case "4": //$NON-NLS-1$
                        Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "3")); //$NON-NLS-1$
                        account.setUnterkonto(StringUtils.EMPTY);
                        account.setMeta(MahoeSynchronizeBackend.BANK_NAME, SantanderBank.NAME);
                        account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "5"); //$NON-NLS-1$

                    default:
                        break;
                    }
                }
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("AbstractAccountMigrator.updatedFailed"), e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("AbstractAccountMigrator.updatedFailedMsg", e.getMessage()), e); //$NON-NLS-1$
        }
    }

    /**
     * Wir mssen nur die Backend-Engine ndern.
     *
     * @param konto
     *            das zu migrierende Konto
     * @throws ApplicationException
     */
    private void migrateAccount(final Konto konto) throws ApplicationException {

        try {
            // Kreditkarten ausklammern
            final KontoType kontoType = KontoType.find(konto.getAccountType());
            if (!KontoType.KREDITKARTE.equals(kontoType)) {
                konto.setUnterkonto(StringUtils.EMPTY);
                konto.setBackendClass("de.frankmuenster.mahoe.MahoeSynchronizeBackend"); //$NON-NLS-1$
                konto.setAccountType(KontoType.GIRO.getValue());
                konto.store();
                // Meta Daten
                konto.setMeta(MahoeSynchronizeBackend.BANK_NAME, SantanderBank.NAME);
                konto.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, SantanderBank.KTO_VER_GIRO);
                // Sync-Options setzen, nur zur Sicherheit
                final SynchronizeOptions sync = new SynchronizeOptions(konto);
                sync.setAutoSaldo(false);
                sync.setSyncAuslandsUeberweisungen(false);
                sync.setSyncKontoauszuege(true);
                sync.setSyncOffline(false);
                sync.setSyncSaldo(true);
                sync.setSyncSepaDauerauftraege(false);
                sync.setSyncSepaLastschriften(false);
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("SantanderAccountMigrator.accountMigrationFailedLog", e.getMessage()), e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("SantanderAccountMigrator.accountMigrationFailed", e.getMessage()), //$NON-NLS-1$
                    e);
        }
    }
}
