/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.santander.web;

import org.apache.commons.lang3.StringUtils;

/**
 * Enum to map and handle Santander Bank month abbreviations
 * 
 * @author Frank Mnster
 * @since 2.4.0
 *
 */
public enum SantanderMonthEnum {

    /**
     * January
     */
    JANUARY(0, "Jan"), //$NON-NLS-1$
    /**
     * February
     */
    FEBRUARY(1, "Feb"), //$NON-NLS-1$
    /**
     * March
     */
    MARCH(2, "Mar"), //$NON-NLS-1$
    /**
     * April
     */
    APRIL(3, "Apr"), //$NON-NLS-1$
    /**
     * May
     */
    MAY(4, "Mai"), //$NON-NLS-1$
    /**
     * June
     */
    JUNE(5, "Jun"), //$NON-NLS-1$
    /**
     * July
     */
    JULY(6, "Jul"), //$NON-NLS-1$
    /**
     * August
     */
    AUGUST(7, "Aug"), //$NON-NLS-1$
    /**
     * September
     */
    SEPTEMBER(8, "Sep"), //$NON-NLS-1$
    /**
     * October
     */
    OCTOBER(9, "Okt"), //$NON-NLS-1$
    /**
     * November
     */
    NOVEMBER(10, "Nov"), //$NON-NLS-1$
    /**
     * December
     */
    DEZEMBER(11, "Dez"); //$NON-NLS-1$

    /** the number of the month in the range from 0-11 */
    private int number;
    /** the short description as used on the Santander web site */
    private String shortDescription;

    /**
     * Return the number of the month of the shortDescription. January is 0.
     *
     * @param shortDescription
     *            the short description as used on the Santander bank web site.
     * @return the month 0 based, or -1 if not found.
     */
    public static int getNumber(final String shortDescription) {

        int number = -1;
        if (StringUtils.trimToNull(shortDescription) != null) {
            for (final SantanderMonthEnum month : SantanderMonthEnum.values()) {
                if (month.getShortDescription().equals(shortDescription)) {
                    number = month.getNumber();
                    break;
                }
            }
        }
        return number;

    }

    SantanderMonthEnum(final int number, final String shortDescription) {
        this.number = number;
        this.shortDescription = shortDescription;
    }

    public int getNumber() {
        return this.number;
    }

    public String getShortDescription() {
        return this.shortDescription;
    }

}
