/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.gui.controller;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.mahoe.Settings;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.FileInput;
import de.willuhn.jameica.hbci.gui.input.KontoInput;

/**
 * Controller for the general settings tab
 *
 * @author Frank Mnster
 */
public class GeneralSettingsController extends AbstractControl implements SettingsController {

    /**
     * FileInput to set the Selenium driver
     */
    private FileInput seleniumDriver;

    /**
     * Auto-close selenium window
     */
    private CheckboxInput seleniumAutoclose;

    /**
     * Constructor with super call
     */
    public GeneralSettingsController() {

        //
        super(null);
    }

    /**
     * Returns a {@link CheckboxInput} to set the Firefox window auto close
     * option.
     *
     * @return {@link CheckboxInput} field
     */
    public CheckboxInput getAutoCloseInput() {
        if (this.seleniumAutoclose == null) {
            this.seleniumAutoclose = new CheckboxInput(Settings.isSeleniumAutoClose());

        }
        return this.seleniumAutoclose;
    }

    /**
     * Returns a {@link FileInput} to select the FireFox driver
     *
     * @return FileInput field
     */
    public FileInput getFileInput() {

        if (this.seleniumDriver == null) {
            final String driver = Settings.getSeleniumDriver();
            this.seleniumDriver = new FileInput(driver, false);
        }
        return this.seleniumDriver;
    }

    /**
     * @see SettingsController#getKonto()
     */
    @Override
    public KontoInput getKonto() {

        return null;
    }

    /**
     * @see SettingsController#speichern()
     */
    @Override
    public void speichern() {

        // path to driver
        String driver = (String) this.getFileInput().getValue();
        if (driver == null) {
            driver = StringUtils.EMPTY;
        }
        Settings.setSeleniumDriver(driver);

        // auto close the FF window after each jop group
        Boolean autoClose = (Boolean) this.getAutoCloseInput().getValue();
        if (autoClose == null) {
            autoClose = false;
        }
        Settings.setSeleniumAutoClose(autoClose);

    }

    /**
     * General settings do not support account statements :-)
     */
    @Override
    public boolean supportsStatements() {

        return false;
    }

}
