/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.jobs;

import java.rmi.RemoteException;
import java.util.LinkedList;
import java.util.List;

import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJob;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJobKontoauszug;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Abstract class of an account provider
 *
 * @author Frank Mnster
 *
 */
public abstract class AbstractMahoeSynchronizeJobProviderKontoauszug extends AbstractMahoeSynchronizeJobProvider {

    /**
     * Standard constructor
     */
    public AbstractMahoeSynchronizeJobProviderKontoauszug() {
        super();
    }

    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    @Override
    public int compareTo(final Object arg0) {

        // Umsaetze und Salden werden zum Schluss ausgefuehrt,
        // damit die oben gesendeten Ueberweisungen gleich mit
        // erscheinen, insofern die Bank das untersttzt.
        return 1;
    }

    /**
     * @see de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider#getSynchronizeJobs(de.willuhn.jameica.hbci.rmi.Konto)
     */
    @Override
    public List<SynchronizeJob> getSynchronizeJobs(final Konto konto) {

        final Class<SynchronizeJobKontoauszug> type = SynchronizeJobKontoauszug.class;

        final List<SynchronizeJob> myJobs = new LinkedList<>();
        for (final Konto kto : this.backend.getSynchronizeKonten(konto)) {
            try {
                // Is Mahoe Backend and account supported?
                if (this.backend.supports(type, kto) && this.supports(type, kto)) {

                    final SynchronizeOptions options = new SynchronizeOptions(kto); // NOPMD
                    // Sync-Option zum Kontoauszugs-Abruf aktiv?
                    if (options.getSyncKontoauszuege()) {
                        // erzeugt eine Instanz von SynchronizeJobKontoauszug
                        final SynchronizeJobKontoauszug job = this.backend.create(type, kto);
                        job.setContext(SynchronizeJob.CTX_ENTITY, kto);
                        myJobs.add(job);
                    }
                }
            } catch (RemoteException | ApplicationException e) {
                Logger.error(I18N.tr("BshSynchronizeJobProviderKontoauszug.getSynchnizedJobsFailed"), e); //$NON-NLS-1$
            }
        }

        return myJobs;
    }

    /**
     * @see AbstractMahoeSynchronizeJobProvider#isJobTypeSupported(Class, Konto)
     */
    @Override
    protected boolean isJobTypeSupported(final Class<? extends SynchronizeJob> type, final Konto account)
            throws RemoteException {

        return SynchronizeJobKontoauszug.class.isAssignableFrom(type);
    }

}
